#!/bin/ksh

##########################################################################
#
# Script Name: run_grid_stat.ksh
#
# Date: 12/9/2013
#
# Description:
#    This script runs the MET/Grid-Stat and MODE tools to verify gridded
#    precipitation forecasts against gridded precipitation analyses.
#    The precipitation fields must first be placed on a common grid prior
#    to running this script.
#
##   This script assumes the output data directory (DATA_ROOT) contains
#    a metprd directory, which is the location of post-processed data
#    that has been run through MET's pcp-combine tool, which buckets the
#    data into desired accumulations. This directory structure can be
#    modified by the user to accommodate their setup.
#
##########################################################################

# Name of this script
SCRIPT=run_grid_stat.ksh

# Make sure $SCRIPTS/constants.ksh exists
if [ ! -x "${SCRIPTS}/constants.ksh" ]; then
  ${ECHO} "ERROR: ${SCRIPTS}/constants.ksh does not exist or is not executable"
  exit 1
fi

# Read constants into the current shell
. ${SCRIPTS}/constants.ksh

# Specify the MET Grid-Stat files to be used
GS_CONFIG_LIST="${MET_CONFIG}/GridStatConfig_${ACCUM_TIME}h"

# Print run parameters
${ECHO}
${ECHO} "${SCRIPT} started at `${DATE}`"
${ECHO}
${ECHO} "    START_TIME = ${START_TIME}"
${ECHO} "FCST_TIME_LIST = ${FCST_TIME_LIST}"
${ECHO} "    ACCUM_TIME = ${ACCUM_TIME}"
${ECHO} "  MET_EXE_ROOT = ${MET_EXE_ROOT}"
${ECHO} "    MET_CONFIG = ${MET_CONFIG}"
${ECHO} "     DATA_ROOT = ${DATA_ROOT}"
${ECHO} "       RAW_OBS = ${RAW_OBS}"
${ECHO} "        CONFIG = ${CONFIG}"

# Make sure $DATA_ROOT exists
if [ ! -d "${DATA_ROOT}" ]; then
  ${ECHO} "ERROR: DATA_ROOT, ${DATA_ROOT} does not exist"
  exit 1
fi

# Make sure RAW_OBS directory exists
if [ ! -d ${RAW_OBS} ]; then
  ${ECHO} "ERROR: RAW_OBS, ${RAW_OBS}, does not exist!"
  exit 1
fi

# Go to working directory
workdir=${DATA_ROOT}/metprd
${MKDIR} -p ${workdir}
cd ${workdir}

# Loop through the forecast times
for FCST_TIME in ${FCST_TIME_LIST}; do

   # Can't verify QPF at the analysis time
   if [[ ${FCST_TIME} -eq 0 ]]; then
     continue
   fi
   
   export FCST_TIME
   
   # Compute the verification date
   FCST_TIME_SEC=`${EXPR} ${FCST_TIME} \* 3600` # convert forecast lead hour to seconds
   YYYY=`${ECHO} ${START_TIME} | ${CUT} -c1-4`  # year (YYYY) of initialization time
   MM=`${ECHO} ${START_TIME} | ${CUT} -c5-6`    # month (MM) of initialization time
   DD=`${ECHO} ${START_TIME} | ${CUT} -c7-8`    # day (DD) of initialization time
   HH=`${ECHO} ${START_TIME} | ${CUT} -c9-10`   # hour (HH) of initialization time
   START_DATE_UT=`${DATE} -ud ''${YYYY}-${MM}-${DD}' UTC '${HH}':00:00' +%s` # convert initialization time to universal time
   VDATE_UT=`${EXPR} ${START_DATE_UT} + ${FCST_TIME_SEC}` # calculate current forecast time in universal time
   VDATE=`${DATE} -ud '1970-01-01 UTC '${VDATE_UT}' seconds' +%Y%m%d%H` # convert universal time to standard time
   VYYYYMMDD=`${ECHO} ${VDATE} | ${CUT} -c1-8`  # forecast time (YYYYMMDD)
   VHH=`${ECHO} ${VDATE} | ${CUT} -c9-10`       # forecast hour (HH)
   ${ECHO} 'valid time for ' ${FCST_TIME} 'h forecast = ' ${VDATE}

   # Get the forecast to verify - forecasts run through MET's pcp-combine tool to provide 03h and 24h accmulations 
   FCST_FILE=${DATA_ROOT}/metprd/wrfpcp_${ACCUM_TIME}_${FCST_TIME}.nc
   if [ ! -e ${FCST_FILE} ]; then
     ${ECHO} "WARNING: Could not find UPP output file: ${FCST_FILE}"
     continue
   fi

   # Get the observation file
   if [[ "${ACCUM_TIME}" == "03" || "${ACCUM_TIME}" == "24" ]]; then
     OBS_FILE=${RAW_OBS}/${VYYYYMMDD}/ST2ml${VYYYYMMDD}${VHH}.${ACCUM_TIME}h.nc
   else
     ${ECHO} "WARNING: Incorrect accumulation time was provided!"
     exit 1
   fi

   if [ ! -e ${OBS_FILE} ]; then
     ${ECHO} "WARNING: Could not find observation file: ${OBS_FILE}"
     continue
   fi

done

   #######################################################################
   #
   #  Run Grid-Stat
   #
   #######################################################################

   for CONFIG_FILE in ${GS_CONFIG_LIST}; do

     # Make sure the Grid-Stat configuration file exists
     if [ ! -e ${CONFIG_FILE} ]; then
       ${ECHO} "WARNING: ${CONFIG_FILE} does not exist!"
       continue
     fi

     ${ECHO} "CALLING: ${MET_EXE_ROOT}/grid_stat ${FCST_FILE} ${OBS_FILE} ${CONFIG_FILE} -outdir . -v 2"

     ${MET_EXE_ROOT}/grid_stat \
       ${FCST_FILE} \
       ${OBS_FILE} \
       ${CONFIG_FILE} \
       -outdir . \
       -v 2

     error=$?
     if [ ${error} -ne 0 ]; then
       ${ECHO} "WARNING: For ${CONFIG}, ${MET_EXE_ROOT}/grid_stat crashed  Exit status: ${error}"
       continue
     fi

   done

##########################################################################

${ECHO} "${SCRIPT} completed at ${DATE}"

exit 0
