#!/bin/ksh

##########################################################################
#
# Script Name: run_met.ksh
#
# Date: 5/21/2012
#
# Description:
#    This script drives the run_grid_stat.ksh (grid-to-grid verif.) and 
#    run_point_stat.ksh (grid-to-point verif.) scripts.  
#
#    This script requires two command line arguements:
#    1) initialization time (YYYYMMDDHH)
#    2) the name of the configuration (e.g., AFWAps or NAMps)
#
#    This script requires the constants.ksh file to be present, which
#    specifies static constants/commands and paths.  The user will need
#    to check and/or modify this file with the appropriate paths.
#
#    This script performs surface and upper-air verification in 3-h
#    increments and QPF verification in 3-h and 24-h increments.
#    
#
##########################################################################

# Name of this script
SCRIPT=run_met.ksh

# Constants
# ===== EDIT HERE ===== #
#
# It is assumed that under the BASE directory, there resides a scripts directory as well as
# an OBS directory.  The OBS directory contains POINT_OBS, PRECIP_03h_OBS, and PRECIP_24h_OBS
# directories.  That is the location of the observation files used for the verification.
# These assumptions can be modified by the user to accommodate their directory structure.
#
BASE=/path/to/base/directory             # path to base/top-level directory
SCRIPTS=${BASE}/scripts                  # path to scripts (run_met.ksh, run_point_stat.ksh, run_grid_stat.ksh)
POINT_OBS=${BASE}/OBS/POINT_OBS          # path to point observations
PRECIP_03_OBS=${BASE}/OBS/PRECIP_03_OBS  # path to 3-hr precip. observations
PRECIP_24_OBS=${BASE}/OBS/PRECIP_24_OBS  # path to 24-hr precip. observations

MET_EXE_ROOT=/path/to/met/executables    # path to MET/bin directory 
MET_CONFIG=/path/to/met/config/files     # path to MET configuration files
MASKS=/path/to/met/masking/regions       # path to masking regions used in verification

# Make sure $SCRIPTS/constants.ksh exists
if [ ! -x "${SCRIPTS}/constants.ksh" ]; then
  ${ECHO} "ERROR: ${SCRIPTS}/constants.ksh does not exist or is not executable"
  exit 1
fi

# Read constants into the current shell
. ${SCRIPTS}/constants.ksh

# The full set of forecast hours to evaluate
FCST_TIME_LIST_03H="000 003 006 009 012 015 018 021 024 027 030 033 036 039 042 045 048 /
051 054 057 060 063 066 069 072 075 078 081 084"
FCST_TIME_LIST_00Z_24H="036 060 084"
FCST_TIME_LIST_12Z_24H="024 048 072"

# Check arguments
if [[ $# -ne 2 ]]; then
   ${ECHO}
   ${ECHO} "Usage: ${0} init config"
   ${ECHO} "   where \"init\"    is the initialization time to be verified (YYYYMMDDHH)."
   ${ECHO} "         \"config\"  is the model to be verified."
   ${ECHO}
   exit 1
fi

# Parse and export command line arguments
START_TIME=${1}
CONFIG=${2}

# Construct the data path and check that it exists
DATA_ROOT=/path/to/output/directory

# Check to see if the directory exists
if [[ ! -e ${DATA_ROOT} ]]; then
   ${ECHO}
   ${ECHO} "ERROR: ${DATA_ROOT} does not exist."
   ${ECHO}
   exit 1
fi

# Export common variables
export BASE
export SCRIPTS
export MASKS
export POINT_OBS
export PRECIP_03_OBS
export PRECIP_24_OBS
export START_TIME
export MET_EXE_ROOT
export MET_CONFIG
export DATA_ROOT
export CONFIG

# Call run_point_stat to verify against NDAS point observations for 3-hourly output
export FCST_TIME_LIST=${FCST_TIME_LIST_03H}
export RAW_OBS=${POINT_OBS}
${SCRIPTS}/run_point_stat.ksh

# Call run_grid_stat to verify against StageII for 3-hourly output
export FCST_TIME_LIST=${FCST_TIME_LIST_03H}
export ACCUM_TIME="03"
export RAW_OBS=${PRECIP_03_OBS}
${SCRIPTS}/run_grid_stat.ksh

# Call run_grid_stat to verify against StageII for 24-hourly output
INIT_HH=`${ECHO} ${START_TIME} | ${CUT} -c9-10`
if [[ ${INIT_HH} -eq 00 ]]; then
   export FCST_TIME_LIST=${FCST_TIME_LIST_00Z_24H}
else
   export FCST_TIME_LIST=${FCST_TIME_LIST_12Z_24H}
fi
export ACCUM_TIME="24"
export RAW_OBS=${PRECIP_24_OBS}
${SCRIPTS}/run_grid_stat.ksh
