#!/bin/ksh

##########################################################################
#
# Script Name: run_point_stat.ksh
#
# Date: 12/9/2013
#
# Description:
#    This script runs the MET Point-Stat tool to verify gridded output
#    using point observations.  The MET PB2NC tool must be run on the
#    PREPBUFR observation files to be used prior to running this script.
#    to running this script.
#
#    This script assumes the output data directory (DATA_ROOT) contains
#    a postprd directory, which is the location of post-processed data.
#    In this script a metprd directory is created under the DATA_ROOT;
#    this directory contains the MET output.  This directory structure
#    can be modified by the user to accommodate their setup.
#
##########################################################################

# Name of this script
SCRIPT=run_point_stat.ksh

# Make sure $SCRIPTS/constants.ksh exists
if [ ! -x "${SCRIPTS}/constants.ksh" ]; then
  ${ECHO} "ERROR: ${SCRIPTS}/constants.ksh does not exist or is not executable"
  exit 1
fi

# Read constants into the current shell
. ${SCRIPTS}/constants.ksh

# Specify the MET Point-Stat configuration files to be used
CONFIG_ADPUPA="${MET_CONFIG}/PointStatConfig_ADPUPA"
CONFIG_ADPSFC="${MET_CONFIG}/PointStatConfig_ADPSFC"
CONFIG_ADPSFC_MPR="${MET_CONFIG}/PointStatConfig_ADPSFC_MPR"
CONFIG_WINDS="${MET_CONFIG}/PointStatConfig_WINDS"

# Print run parameters
${ECHO}
${ECHO} "${SCRIPT} started at `${DATE}`"
${ECHO}
${ECHO} "    START_TIME = ${START_TIME}"
${ECHO} "FCST_TIME_LIST = ${FCST_TIME_LIST}"
${ECHO} "  MET_EXE_ROOT = ${MET_EXE_ROOT}"
${ECHO} "    MET_CONFIG = ${MET_CONFIG}"
${ECHO} "     DATA_ROOT = ${DATA_ROOT}"
${ECHO} "       RAW_OBS = ${RAW_OBS}"
${ECHO} "        CONFIG = ${CONFIG}"

# Make sure $DATA_ROOT exists
if [ ! -d "${DATA_ROOT}" ]; then
  ${ECHO} "ERROR: DATA_ROOT, ${DATA_ROOT} does not exist"
  exit 1
fi

# Make sure $DATA_ROOT/postprd exists
if [ ! -d "${DATA_ROOT}/postprd" ]; then
  ${ECHO} "ERROR: DATA_ROOT/postprd, ${DATA_ROOT}/postprd does not exist"
  exit 1
fi

# Make sure RAW_OBS directory exists
if [ ! -d ${RAW_OBS} ]; then
  ${ECHO} "ERROR: RAW_OBS, ${RAW_OBS}, does not exist!"
  exit 1
fi

# Make sure the Point-Stat configuration files exists
if [ ! -e ${CONFIG_ADPUPA} ]; then
  ${ECHO} "ERROR: ${CONFIG_ADPUPA} does not exist!"
  exit 1
fi
if [ ! -e ${CONFIG_ADPSFC} ]; then
  ${ECHO} "ERROR: ${CONFIG_ADPSFC} does not exist!"
  exit 1
fi
if [ ! -e ${CONFIG_ADPSFC_MPR} ]; then
  ${ECHO} "ERROR: ${CONFIG_ADPSFC_MPR} does not exist!"
  exit 1
fi
if [ ! -e ${CONFIG_WINDS} ]; then
  ${ECHO} "ERROR: ${CONFIG_WINDS} does not exist!"
  exit 1
fi

# Go to working directory
workdir=${DATA_ROOT}/metprd
${MKDIR} -p ${workdir}
cd ${workdir}

# Loop through the forecast times
for FCST_TIME in ${FCST_TIME_LIST}; do

   export FCST_TIME

   # Compute the verification date
   FCST_TIME_SEC=`${EXPR} ${FCST_TIME} \* 3600` # convert forecast lead hour to seconds
   YYYY=`${ECHO} ${START_TIME} | ${CUT} -c1-4`  # year (YYYY) of initialization time
   MM=`${ECHO} ${START_TIME} | ${CUT} -c5-6`    # month (MM) of initialization time
   DD=`${ECHO} ${START_TIME} | ${CUT} -c7-8`    # day (DD) of initialization time
   HH=`${ECHO} ${START_TIME} | ${CUT} -c9-10`   # hour (HH) of initialization time
   START_DATE_UT=`${DATE} -ud ''${YYYY}-${MM}-${DD}' UTC '${HH}':00:00' +%s` # convert initialization time to universal time
   VDATE_UT=`${EXPR} ${START_DATE_UT} + ${FCST_TIME_SEC}` # calculate current forecast time in universal time
   VDATE=`${DATE} -ud '1970-01-01 UTC '${VDATE_UT}' seconds' +%Y%m%d%H` # convert universal time to standard time
   VYYYYMMDD=`${ECHO} ${VDATE} | ${CUT} -c1-8`  # forecast time (YYYYMMDD)
   VHH=`${ECHO} ${VDATE} | ${CUT} -c9-10`       # forecast hour (HH)
   ${ECHO} 'valid time for ' ${FCST_TIME} 'h forecast = ' ${VDATE}

   # Get the forecast to verify
   FCST_FILE=${DATA_ROOT}/postprd/wrfprs_${FCST_TIME}.tm00
   if [ ! -e ${FCST_FILE} ]; then
     ${ECHO} "WARNING: Could not find UPP output file: ${FCST_FILE}"
     continue
   fi

   # Get the observation file
   OBS_FILE=`${LS} -1 ${RAW_OBS}/${VYYYYMMDD}/prepbufr.ndas.${VYYYYMMDD}.t${VHH}z.*.nc | ${HEAD} -1`
   if [ ! -e ${OBS_FILE} ]; then
     ${ECHO} "WARNING: Could not find observation file: ${OBS_FILE}"
     continue
   fi

   #######################################################################
   #
   #  Run Point-Stat
   #
   #######################################################################

   # Verify upper air variables only at 00Z and 12Z
   if [ "${VHH}" == "00" -o "${VHH}" == "12" ]; then
     CONFIG_FILE=${CONFIG_ADPUPA}
     ${MET_EXE_ROOT}/point_stat ${FCST_FILE} ${OBS_FILE} ${CONFIG_FILE} \
       -outdir . -v 2

     error=$?
     if [ ${error} -ne 0 ]; then
       ${ECHO} "WARNING: For ${CONFIG}, ${MET_EXE_ROOT}/point_stat ${CONFIG_FILE} crashed  Exit status: ${error}"
       continue
     fi
   fi

   # Verify surface variables for each forecast hour
   CONFIG_FILE=${CONFIG_ADPSFC}

   ${ECHO} "CALLING: ${MET_EXE_ROOT}/point_stat ${FCST_FILE} ${OBS_FILE} ${CONFIG_FILE} -outdir . -v 2"

   ${MET_EXE_ROOT}/point_stat ${FCST_FILE} ${OBS_FILE} ${CONFIG_FILE} \
     -outdir . -v 2

   error=$?
   if [ ${error} -ne 0 ]; then
     ${ECHO} "WARNING: For ${CONFIG}, ${MET_EXE_ROOT}/point_stat ${CONFIG_FILE} crashed  Exit status: ${error}"
     continue
   fi

   # Verify surface variables for each forecast hour - matched pair output
   CONFIG_FILE=${CONFIG_ADPSFC_MPR}

   ${ECHO} "CALLING: ${MET_EXE_ROOT}/point_stat ${FCST_FILE} ${OBS_FILE} ${CONFIG_FILE} -outdir . -v 2"

   ${MET_EXE_ROOT}/point_stat ${FCST_FILE} ${OBS_FILE} ${CONFIG_FILE} \
     -outdir . -v 2

   error=$?
   if [ ${error} -ne 0 ]; then
     ${ECHO} "WARNING: For ${CONFIG}, ${MET_EXE_ROOT}/point_stat ${CONFIG_FILE} crashed  Exit status: ${error}"
     continue
   fi

   # Verify winds for each forecast hour
   CONFIG_FILE=${CONFIG_WINDS}

   ${ECHO} "CALLING: ${MET_EXE_ROOT}/point_stat ${FCST_FILE} ${OBS_FILE} ${CONFIG_FILE} -outdir . -v 2"

   ${MET_EXE_ROOT}/point_stat ${FCST_FILE} ${OBS_FILE} ${CONFIG_FILE} \
     -outdir . -v 2

   error=$?
   if [ ${error} -ne 0 ]; then
     ${ECHO} "WARNING: For ${CONFIG}, ${MET_EXE_ROOT}/point_stat ${CONFIG_FILE} crashed  Exit status: ${error}"
     continue
   fi

done

##########################################################################

${ECHO} "${SCRIPT} completed at ${DATE}"

exit 0

